require "spec_helper"
require "rampart/proxy"

describe Rampart::Proxy do
  describe "#authenticate!" do
    it "returns user when successful" do
      user = Object.new
      
      strategy_manager = define_strategy_manager
      strategy_manager.register(:basic, define_succeeding_strategy(user))

      user_store = MiniTest::Mock.new
      user_store.expect(:store, true, [Object])

      session = MiniTest::Mock.new 

      proxy = Rampart::Proxy.new(strategy_manager, session, user_store)
      proxy.authenticate!(:basic).must_equal(user)
    end

    it "throws rack truple when not successful" do
      user = Object.new
      
      strategy_manager = define_strategy_manager
      strategy_manager.register(:basic, define_failing_strategy)

      user_store = MiniTest::Mock.new
      user_store.expect(:store, true, [Object])

      session = MiniTest::Mock.new 

      proxy = Rampart::Proxy.new(strategy_manager, session, user_store)
      response = catch(:rampart){ proxy.authenticate!(:basic) }
      response.must_be_instance_of(Array)
    end
  end

  describe "#authenticate" do
    it "returns user when successful" do
      user = Object.new
      
      strategy_manager = define_strategy_manager
      strategy_manager.register(:basic, define_succeeding_strategy(user))

      user_store = MiniTest::Mock.new
      user_store.expect(:store, true, [Object])

      session = MiniTest::Mock.new

      proxy = Rampart::Proxy.new(strategy_manager, session, user_store)
      proxy.authenticate(:basic).must_equal(user)
    end

    it "returns nil when not successful" do
      user = Object.new
      
      strategy_manager = define_strategy_manager
      strategy_manager.register(:basic, define_failing_strategy)

      user_store = MiniTest::Mock.new
      user_store.expect(:store, true, [Object])

      session = MiniTest::Mock.new 
      
      proxy = Rampart::Proxy.new(strategy_manager, session, user_store)
      proxy.authenticate(:basic).must_be_nil
    end
  end
end