require "spec_helper"
require "rampart/strategy_manager"

describe Rampart::StrategyManager do
  describe "register" do
    it "can register a strategy" do
      strategy = Object.new
      manager = define_strategy_manager
      manager.register(:basic, strategy)
    end

    it "complains if strategy name registered more than once" do
      strategy = Object.new
      manager = define_strategy_manager
      manager.register(:basic, strategy)
      proc {
        manager.register(:basic, strategy)
      }.must_raise(ArgumentError)
    end
  end

  describe "authenticate!" do

    it "complains when strategy name has not been registered" do
      manager = define_strategy_manager
      proc {
        manager.authenticate!(:my_strategy_name)
      }.must_raise(ArgumentError)
    end

    it "returns fail event when no strategy names given" do
      manager = define_strategy_manager
      manager.authenticate!.must_be_instance_of(Rampart::FailEvent)
    end

    describe "single strategy" do

      it "returns fail event when strategy passes" do
        strategy = define_passing_strategy
        manager = define_strategy_manager
        manager.register(:strategy1, strategy)
        manager.authenticate!(:strategy1).must_be_instance_of(Rampart::FailEvent)
      end

      it "returns fail event when strategy fails" do
        strategy = define_failing_strategy
        manager = define_strategy_manager
        manager.register(:strategy1, strategy)
        manager.authenticate!(:strategy1).must_be_instance_of(Rampart::FailEvent)
      end

      it "returns redirect event when strategy redirects" do
        strategy = define_redirecting_strategy
        manager = define_strategy_manager
        manager.register(:strategy1, strategy)
        manager.authenticate!(:strategy1).must_be_instance_of(Rampart::RedirectEvent)
      end

      it "returns success event when strategy succeeds" do
        strategy = define_succeeding_strategy
        manager = define_strategy_manager
        manager.register(:strategy1, strategy)
        manager.authenticate!(:strategy1).must_be_instance_of(Rampart::SuccessEvent)
      end

      it "returns fail event when strategy not valid" do
        strategy = define_invalid_strategy
        manager = define_strategy_manager
        manager.register(:strategy1, strategy)
        manager.authenticate!(:strategy1).must_be_instance_of(Rampart::FailEvent)
      end

    end # single strategy

    describe "multple strategies" do
      it "returns first halting strategy" do
        manager = define_strategy_manager
        manager.register(:strategy1, define_passing_strategy)
        manager.register(:strategy2, define_invalid_strategy)
        manager.register(:strategy3, define_redirecting_strategy)
        manager.register(:strategy4, define_succeeding_strategy)
        manager.authenticate!(:strategy1, :strategy2, :strategy3, :strategy4).must_be_instance_of(Rampart::RedirectEvent)
      end

      it "returns fail when no valid halting strategies found" do
        manager = define_strategy_manager
        manager.register(:strategy1, define_passing_strategy)
        manager.register(:strategy2, define_invalid_strategy)
        manager.authenticate!(:strategy1, :strategy2).must_be_instance_of(Rampart::FailEvent)
      end
    end
  end
end